"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Type

from encommon.times import Timer
from encommon.types import NCNone

from .params import AutoNickPluginParams
from ..status import StatusPlugin
from ..status import StatusPluginStates
from ...robie.childs import RobiePlugin



class AutoNickPlugin(RobiePlugin):
    """
    Integrate with the Robie routine and perform operations.

    .. note::
       This plugin maintains configured nickname on server.
    """

    __started: bool

    __timer: Timer


    def __post__(
        self,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        self.__started = False

        params = self.params

        self.__timer = Timer(
            params.interval)

        self.__status('pending')


    def validate(
        self,
    ) -> None:
        """
        Perform advanced validation on the parameters provided.
        """

        # Review the parameters


    @classmethod
    def schema(
        cls,
    ) -> Type[AutoNickPluginParams]:
        """
        Return the configuration parameters relevant for class.

        :returns: Configuration parameters relevant for class.
        """

        return AutoNickPluginParams


    @property
    def params(
        self,
    ) -> AutoNickPluginParams:
        """
        Return the Pydantic model containing the configuration.

        :returns: Pydantic model containing the configuration.
        """

        params = super().params

        assert isinstance(
            params,
            AutoNickPluginParams)

        return params


    def operate(
        self,
    ) -> None:
        """
        Perform the operation related to Robie service threads.
        """

        assert self.thread

        thread = self.thread
        mqueue = thread.mqueue
        timer = self.__timer


        if not self.__started:
            self.__started = True
            self.__status('normal')


        if timer.ready():
            self.__operate()


        while not mqueue.empty:
            mqueue.get()


    def __operate(
        self,
    ) -> None:
        """
        Perform the operation related to Robie service threads.
        """

        from ...clients import IRCClient

        assert self.thread

        thread = self.thread
        member = thread.member
        cqueue = member.cqueue
        params = self.params

        clients = (
            thread.service
            .clients.childs
            .values())

        names = params.clients

        failure: set[bool] = set()


        for client in clients:

            name = client.name

            # Ignore unrelated clients
            if name not in names:
                continue

            assert isinstance(
                client, IRCClient)

            connected = (
                client.client
                .connected)

            # Bypass when disconnected
            if connected is False:
                failure.add(True)
                return None


            should = (
                client.params
                .client.nickname)

            current = (
                client.client
                .nickname)

            if current == should:
                return None


            failure.add(True)

            rawcmd = f'NICK :{should}'

            client.put_command(
                cqueue, rawcmd)


        self.__status(
            'failure'
            if any(failure)
            else 'normal')


    def __status(
        self,
        status: StatusPluginStates,
    ) -> None:
        """
        Update or insert the status of the Robie child instance.

        :param status: One of several possible value for status.
        """

        thread = self.thread
        params = self.params

        if thread is None:
            return None

        plugins = (
            thread.service
            .plugins.childs)

        if 'status' not in plugins:
            return NCNone

        plugin = plugins['status']

        assert isinstance(
            plugin, StatusPlugin)

        (plugin.update(
            unique=self.name,
            group='Management',
            title='IRC',
            icon=params.status,
            state=status))



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from copy import deepcopy
from pathlib import Path
from threading import Thread
from time import sleep as block_sleep
from typing import TYPE_CHECKING

from encommon.types import getate
from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs
from encommon.types import setate

from enconnect.fixtures import DSCClientSocket
from enconnect.fixtures import IRCClientSocket
from enconnect.fixtures import MTMClientSocket

from httpx import Response

from respx import MockRouter

from ..plugin import NagiosPlugin
from ....clients.discord.test import DSCEVENTS
from ....clients.discord.test import DSCEVENT_HUBERT_CHAN
from ....clients.discord.test import DSCEVENT_HUBERT_PRIV
from ....clients.irc.test import IRCEVENTS
from ....clients.irc.test import IRCEVENT_HUBERT_CHAN
from ....clients.irc.test import IRCEVENT_HUBERT_PRIV
from ....clients.mattermost.test import MTMEVENTS
from ....clients.mattermost.test import MTMEVENT_HUBERT_CHAN
from ....clients.mattermost.test import MTMEVENT_HUBERT_PRIV
from ....conftest import config_factory
from ....conftest import robie_factory
from ....conftest import service_factory

if TYPE_CHECKING:
    from ....robie import RobieService



def test_NagiosPlugin(
    service: 'RobieService',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    """

    plugin = (
        service.plugins
        .childs['nagios'])

    assert isinstance(
        plugin, NagiosPlugin)


    attrs = lattrs(plugin)

    assert attrs == [
        '_RobieChild__robie',
        '_RobieChild__name',
        '_RobieChild__params',
        '_NagiosPlugin__started',
        '_NagiosPlugin__current',
        '_RobiePlugin__thread']


    assert inrepr(
        'plugin.NagiosPlugin',
        plugin)

    assert isinstance(
        hash(plugin), int)

    assert instr(
        'plugin.NagiosPlugin',
        plugin)


    plugin.validate()

    assert plugin.robie

    assert plugin.enable

    assert plugin.name == 'nagios'

    assert plugin.kind == 'plugin'

    assert plugin.schema()

    assert plugin.params

    assert plugin.thread

    assert plugin.dumped

    assert plugin.current



def test_NagiosPlugin_cover(  # noqa: CFQ001
    tmp_path: Path,
    client_dscsock: DSCClientSocket,
    client_ircsock: IRCClientSocket,
    client_mtmsock: MTMClientSocket,
    respx_mock: MockRouter,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param tmp_path: pytest object for temporal filesystem.
    :param client_dscsock: Object to mock client connection.
    :param client_ircsock: Object to mock client connection.
    :param client_mtmsock: Object to mock client connection.
    :param respx_mock: Object for mocking request operation.
    """

    robie = robie_factory(
        config_factory(tmp_path))

    service = (
        service_factory(robie))


    childs = robie.childs
    plugins = childs.plugins

    plugin = plugins['nagios']

    assert isinstance(
        plugin, NagiosPlugin)


    dscevents = (
        deepcopy(DSCEVENTS))

    dscchan = deepcopy(
        DSCEVENT_HUBERT_CHAN)

    dscpriv = deepcopy(
        DSCEVENT_HUBERT_PRIV)

    setate(
        dscchan,
        'd/content',
        '!nagios')

    setate(
        dscpriv,
        'd/content',
        '!nagios')

    dscevents.append(dscpriv)
    dscevents.append(dscchan)

    client_dscsock(dscevents)


    ircevents = (
        deepcopy(IRCEVENTS))

    ircchan = (
        IRCEVENT_HUBERT_CHAN)

    ircpriv = (
        IRCEVENT_HUBERT_PRIV)

    ircchan = (
        ircchan.replace(
            ':ircbot',
            ':!nagios'))

    ircpriv = (
        ircpriv.replace(
            ':ircbot',
            ':!nagios'))

    ircevents.append(ircpriv)
    ircevents.append(ircchan)

    client_ircsock(ircevents)


    mtmevents = (
        deepcopy(MTMEVENTS))

    mtmchan = deepcopy(
        MTMEVENT_HUBERT_CHAN)

    mtmpriv = deepcopy(
        MTMEVENT_HUBERT_PRIV)

    post = 'data/post'

    setate(
        mtmchan, post,
        (getate(mtmchan, post)
         .replace(
             'mtmbot',
             '!nagios')))

    setate(
        mtmpriv, post,
        (getate(mtmpriv, post)
         .replace(
             'mtmbot',
             '!nagios')))

    mtmevents.append(mtmpriv)
    mtmevents.append(mtmchan)

    client_mtmsock(mtmevents)


    (respx_mock
     .get(
         'https://mocked:443'
         '/cgi-bin/statusjson.cgi'
         '?query=servicelist')
     .mock(Response(
         status_code=200)))

    (respx_mock
     .get(
         'https://mocked:443'
         '/cgi-bin/statusjson.cgi'
         '?query=hostlist')
     .mock(Response(
         status_code=200)))


    client_dscsock(dscevents)
    client_ircsock(ircevents)
    client_mtmsock(mtmevents)


    service.limit(
        plugins=[
            'nagios',
            'status'])

    service.start()


    thread = Thread(
        target=service.operate)

    thread.start()

    block_sleep(5)

    service.soft()

    while service.running:
        block_sleep(1)

    service.stop()

    thread.join()



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from argparse import ArgumentParser
from sys import argv as sys_argv
from sys import stdin
from typing import Optional

from encommon.types import DictStrAny
from encommon.types.strings import SEMPTY
from encommon.utils import print_ansi

from ..robie import RobieConfig



def arguments(
    args: Optional[list[str]] = None,
) -> DictStrAny:
    """
    Construct arguments which are associated with the file.

    :param args: Override the source for the main arguments.
    :returns: Construct arguments from command line options.
    """

    parser = ArgumentParser()

    args = args or sys_argv[1:]


    parser.add_argument(
        '--config',
        required=True,
        help=(
            'complete or relative '
            'path to config file'))


    parser.add_argument(
        '--console',
        action='store_true',
        default=False,
        help=(
            'write log messages '
            'to standard output'))


    parser.add_argument(
        '--debug',
        action='store_true',
        default=False,
        help=(
            'increase logging level '
            'for standard output'))


    parser.add_argument(
        '--encrypt',
        help=(
            'encryption operation '
            'that is performed'))


    parser.add_argument(
        '--decrypt',
        action='store_true',
        help=(
            'encryption operation '
            'that is performed'))


    return vars(
        parser
        .parse_args(args))



def operation(
    # NOCVR
    config: RobieConfig,
) -> None:
    """
    Perform whatever operation is associated with the file.

    :param config: Primary class instance for configuration.
    """

    sargs = config.sargs
    crypts = config.crypts

    encrypt = sargs['encrypt']
    decrypt = sargs['decrypt']

    _encrypt = crypts.encrypt
    _decrypt = crypts.decrypt

    dashes = '─' * 80

    value: Optional[str] = None


    if encrypt is not None:

        print_ansi(
            'Use <c96>Ctrl-D<c0> '
            'on newline to exit, or '
            'twice on one without')

        print_ansi(
            f'<c32>{dashes}<c0>')

        provide = stdin.read()

        if provide[-1] != '\n':
            print_ansi(SEMPTY)

        print_ansi(
            f'<c32>{dashes}<c0>')

        value = _encrypt(
            value=provide,
            unique=encrypt)

    elif decrypt is not None:

        print_ansi(
            'Paste <c96>encrypted<c0> '
            'crypts string value')

        value = _decrypt(
            value=input())


    assert value is not None

    value = value.rstrip('\n')

    print_ansi(f'<c31>{dashes}<c0>')
    print_ansi(f'<c91>{value}<c0>')
    print_ansi(f'<c31>{dashes}<c0>')



def execution(
    # NOCVR
    args: Optional[list[str]] = None,
) -> None:
    """
    Perform whatever operation is associated with the file.

    :param args: Override the source for the main arguments.
    """

    config = RobieConfig(
        arguments(args))

    config.logger.start()

    config.logger.log_i(
        base='execution/crypts',
        status='started')

    operation(config)

    config.logger.log_i(
        base='execution/crypts',
        status='stopped')

    config.logger.stop()



if __name__ == '__main__':
    execution()  # NOCVR



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated
from typing import Any
from typing import Callable
from typing import Optional

from pydantic import Field

from ..status import StatusPluginIconParams
from ...robie.params import RobiePluginParams



class AutoNickPluginParams(RobiePluginParams, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    clients: Annotated[
        list[str],
        Field(...,
              description='List of clients to enable plugin',
              min_length=1)]

    interval: Annotated[
        int,
        Field(5,
              description='Interval when nick is validated',
              ge=5, le=300)]

    status: Annotated[
        StatusPluginIconParams,
        Field(default_factory=StatusPluginIconParams,
              description='Icon used per chat platform')]


    def __init__(
        # NOCVR
        self,
        /,
        _parse: Optional[Callable[..., Any]] = None,
        **data: Any,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """


        clients = data.get('clients')

        if isinstance(clients, str):
            data['clients'] = [clients]


        if _parse is not None:

            parsable = [
                'clients',
                'interval',
                'status']

            for key in parsable:

                value = data.get(key)

                if value is None:
                    continue

                value = _parse(value)

                data[key] = value


        super().__init__(**data)



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from .member import RobieMember
from ..threads import RobieClientThread

if TYPE_CHECKING:
    from .member import RobieThreads
    from ..childs import RobieClient  # noqa: F401



_CLIENTS = dict[str, 'RobieClient']



class RobieClients(RobieMember):
    """
    Common methods and routines for Chatting Robie members.
    """


    @property
    def childs(
        self,
    ) -> _CLIENTS:
        """
        Return the value for the attribute from class instance.

        .. note::
           Deviates from enhomie in build happens downstream,
           because all of the enhomie members are origin based
           where enrobie children do not have consistent base.

        :returns: Value for the attribute from class instance.
        """

        clients: _CLIENTS = {}

        threads = (
            self.threads
            .values())


        for thread in threads:

            assert isinstance(
                thread,
                RobieClientThread)

            client = thread.client
            name = client.name

            clients[name] = client


        return clients


    def operate(
        self,
    ) -> None:
        """
        Perform the operation related to Robie service members.
        """

        # Spread commands to clients

        robie = self.robie
        childs = robie.childs
        clients = childs.clients
        vacate = self.vacate
        cqueue = self.cqueue

        threads = (
            self.threads
            .values())


        def _put_command() -> None:

            child = thread.child
            _name = child.name

            if name != _name:
                return None

            (thread
             .cqueue.put(citem))


        while not cqueue.empty:

            citem = cqueue.get()

            if vacate.is_set():
                continue

            client = clients[
                citem.client]

            name = client.name

            for thread in threads:
                _put_command()


    def build_threads(
        self,
    ) -> None:
        """
        Construct instances using the configuration parameters.

        .. note::
           Deviates from enhomie in build happens downstream,
           because all of the enhomie members are origin based
           where enrobie children do not have consistent base.
        """

        robie = self.robie
        childs = robie.childs
        clients = childs.clients

        model = RobieClientThread


        threads: 'RobieThreads' = {}


        items = clients.items()

        for name, client in items:

            if not client.enable:
                continue  # NOCVR

            object = model(
                self, client)

            client.thread = object
            threads[name] = object


        self.threads = threads



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated
from typing import Any
from typing import Callable
from typing import Optional

from enconnect.mattermost import ClientParams

from pydantic import Field

from ...plugins import StatusPluginIconParams
from ...robie.params import RobieClientParams



class MTMClientParams(RobieClientParams, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    client: Annotated[
        ClientParams,
        Field(...,
              description='Parameters for the base client')]

    delay: Annotated[
        int,
        Field(15,
              description='Period to wait for reconnect',
              ge=1, le=300)]

    status: Annotated[
        StatusPluginIconParams,
        Field(default_factory=StatusPluginIconParams,
              description='Icon used per chat platform')]


    def __init__(
        # NOCVR
        self,
        /,
        _parse: Optional[Callable[..., Any]] = None,
        **data: Any,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """


        if _parse is not None:

            parsable = [
                'client',
                'delay',
                'status']

            for key in parsable:

                value = data.get(key)

                if value is None:
                    continue

                value = _parse(value)

                data[key] = value


        super().__init__(**data)
